@@ -1,4 +1,5 @@
 """Generic interface for least-square minimization."""
+from __future__ import division, print_function, absolute_import
 
 from warnings import warn
 
@@ -232,17 +233,17 @@ def construct_loss_function(m, loss, f_scale):
 
 def least_squares(
         fun, x0, jac='2-point', bounds=(-np.inf, np.inf), method='trf',
-        ftol=EPS**0.5, xtol=EPS**0.5, gtol=EPS**0.5, x_scale=1.0,
-        loss='linear', f_scale=1.0, diff_step=None, tr_solver=None,
-        tr_options={}, jac_sparsity=None, max_nfev=None, verbose=0, args=(),
-        kwargs={}):
+        ftol=1e-8, xtol=1e-8, gtol=1e-8, x_scale=1.0, loss='linear',
+        f_scale=1.0, diff_step=None, tr_solver=None, tr_options={},
+        jac_sparsity=None, max_nfev=None, verbose=0, args=(), kwargs={}):
     """Solve a nonlinear least-squares problem with bounds on the variables.
 
-    Given the residuals f(x) (an m-dimensional function of n variables) and
-    the loss function rho(s) (a scalar function), `least_squares` finds a
-    local minimum of the cost function F(x)::
+    Given the residuals f(x) (an m-dimensional real function of n real
+    variables) and the loss function rho(s) (a scalar function), `least_squares`
+    finds a local minimum of the cost function F(x)::
 
-        F(x) = 0.5 * sum(rho(f_i(x)**2), i = 1, ..., m), lb <= x <= ub
+        minimize F(x) = 0.5 * sum(rho(f_i(x)**2), i = 0, ..., m - 1)
+        subject to lb <= x <= ub
 
     The purpose of the loss function rho(s) is to reduce the influence of
     outliers on the solution.
@@ -254,7 +255,10 @@ def least_squares(
         ``fun(x, *args, **kwargs)``, i.e., the minimization proceeds with
         respect to its first argument. The argument ``x`` passed to this
         function is an ndarray of shape (n,) (never a scalar, even for n=1).
-        It must return a 1-d array_like of shape (m,) or a scalar.
+        It must return a 1-d array_like of shape (m,) or a scalar. If the
+        argument ``x`` is complex or the function ``fun`` returns complex
+        residuals, it must be wrapped in a real function of real arguments,
+        as shown at the end of the Examples section.
     x0 : array_like with shape (n,) or float
         Initial guess on independent variables. If float, it will be treated
         as a 1-d array with one element.
@@ -263,15 +267,14 @@ def least_squares(
         element (i, j) is the partial derivative of f[i] with respect to
         x[j]). The keywords select a finite difference scheme for numerical
         estimation. The scheme '3-point' is more accurate, but requires
-        twice as much operations compared to '2-point' (default). The
-        scheme 'cs' uses complex steps, and while potentially the most
-        accurate, it is applicable only when `fun` correctly handles
-        complex inputs and can be analytically continued to the complex
-        plane. Method 'lm' always uses the '2-point' scheme. If callable,
-        it is used as ``jac(x, *args, **kwargs)`` and should return a
-        good approximation (or the exact value) for the Jacobian as an
-        array_like (np.atleast_2d is applied), a sparse matrix or a
-        `scipy.sparse.linalg.LinearOperator`.
+        twice as many operations as '2-point' (default). The scheme 'cs'
+        uses complex steps, and while potentially the most accurate, it is
+        applicable only when `fun` correctly handles complex inputs and
+        can be analytically continued to the complex plane. Method 'lm'
+        always uses the '2-point' scheme. If callable, it is used as
+        ``jac(x, *args, **kwargs)`` and should return a good approximation
+        (or the exact value) for the Jacobian as an array_like (np.atleast_2d
+        is applied), a sparse matrix or a `scipy.sparse.linalg.LinearOperator`.
     bounds : 2-tuple of array_like, optional
         Lower and upper bounds on independent variables. Defaults to no bounds.
         Each array must match the size of `x0` or be a scalar, in the latter
@@ -291,14 +294,13 @@ def least_squares(
 
         Default is 'trf'. See Notes for more information.
     ftol : float, optional
-        Tolerance for termination by the change of the cost function.
-        Default is the square root of machine epsilon. The optimization process
-        is stopped when ``dF < ftol * F``, and there was an adequate agreement
-        between a local quadratic model and the true model in the last step.
+        Tolerance for termination by the change of the cost function. Default
+        is 1e-8. The optimization process is stopped when  ``dF < ftol * F``,
+        and there was an adequate agreement between a local quadratic model and
+        the true model in the last step.
     xtol : float, optional
         Tolerance for termination by the change of the independent variables.
-        Default is the square root of machine epsilon. The exact condition
-        checked depends on the `method` used:
+        Default is 1e-8. The exact condition depends on the `method` used:
 
             * For 'trf' and 'dogbox' : ``norm(dx) < xtol * (xtol + norm(x))``
             * For 'lm' : ``Delta < xtol * norm(xs)``, where ``Delta`` is
@@ -306,9 +308,8 @@ def least_squares(
               scaled according to `x_scale` parameter (see below).
 
     gtol : float, optional
-        Tolerance for termination by the norm of the gradient. Default is
-        the square root of machine epsilon. The exact condition depends
-        on a `method` used:
+        Tolerance for termination by the norm of the gradient. Default is 1e-8.
+        The exact condition depends on a `method` used:
 
             * For 'trf' : ``norm(g_scaled, ord=np.inf) < gtol``, where
               ``g_scaled`` is the value of the gradient scaled to account for
@@ -323,9 +324,9 @@ def least_squares(
     x_scale : array_like or 'jac', optional
         Characteristic scale of each variable. Setting `x_scale` is equivalent
         to reformulating the problem in scaled variables ``xs = x / x_scale``.
-        An alternative view is that the size of a trust-region along j-th
+        An alternative view is that the size of a trust region along j-th
         dimension is proportional to ``x_scale[j]``. Improved convergence may
-        be achieved by setting `x_scale` such that a step of a given length
+        be achieved by setting `x_scale` such that a step of a given size
         along any of the scaled variables has a similar effect on the cost
         function. If set to 'jac', the scale is iteratively updated using the
         inverse norms of the columns of the Jacobian matrix (as described in
@@ -338,7 +339,7 @@ def least_squares(
             * 'soft_l1' : ``rho(z) = 2 * ((1 + z)**0.5 - 1)``. The smooth
               approximation of l1 (absolute value) loss. Usually a good
               choice for robust least squares.
-            * 'huber' : ``rho(z) = z if z <= 1 else z**0.5 - 1``. Works
+            * 'huber' : ``rho(z) = z if z <= 1 else 2*z**0.5 - 1``. Works
               similarly to 'soft_l1'.
             * 'cauchy' : ``rho(z) = ln(1 + z)``. Severely weakens outliers
               influence, but may cause difficulties in optimization process.
@@ -385,7 +386,7 @@ def least_squares(
               least-squares problem and only requires matrix-vector product
               evaluations.
 
-        If None (default) the solver is chosen based on type of Jacobian
+        If None (default) the solver is chosen based on the type of Jacobian
         returned on the first iteration.
     tr_options : dict, optional
         Keyword options passed to trust-region solver.
@@ -394,17 +395,18 @@ def least_squares(
             * ``tr_solver='lsmr'``: options for `scipy.sparse.linalg.lsmr`.
               Additionally  ``method='trf'`` supports  'regularize' option
               (bool, default is True) which adds a regularization term to the
-              normal equations, which improves convergence if Jacobian is
+              normal equation, which improves convergence if the Jacobian is
               rank-deficient [Byrd]_ (eq. 3.4).
 
     jac_sparsity : {None, array_like, sparse matrix}, optional
         Defines the sparsity structure of the Jacobian matrix for finite
-        differences. If the Jacobian has only few non-zeros in *each* row,
-        providing the sparsity structure will greatly speed up the computations
-        [Curtis]_. Should have shape (m, n). A zero entry means that a
-        corresponding element in the Jacobian is identically zero. If provided,
-        forces the use of 'lsmr' trust-region solver. If None (default) then
-        dense differencing will be used. Has no effect for 'lm' method.
+        difference estimation, its shape must be (m, n). If the Jacobian has
+        only few non-zero elements in *each* row, providing the sparsity
+        structure will greatly speed up the computations [Curtis]_. A zero
+        entry means that a corresponding element in the Jacobian is identically
+        zero. If provided, forces the use of 'lsmr' trust-region solver.
+        If None (default) then dense differencing will be used. Has no effect
+        for 'lm' method.
     verbose : {0, 1, 2}, optional
         Level of algorithm's verbosity:
 
@@ -561,7 +563,7 @@ def least_squares(
     Examples
     --------
     In this example we find a minimum of the Rosenbrock function without bounds
-    on independed variables.
+    on independent variables.
 
     >>> def fun_rosenbrock(x):
     ...     return np.array([10 * (x[1] - x[0]**2), (1 - x[0])])
@@ -576,9 +578,9 @@ def least_squares(
     >>> res_1.x
     array([ 1.,  1.])
     >>> res_1.cost
-    2.4651903288156619e-30
+    9.8669242910846867e-30
     >>> res_1.optimality
-    4.4408921315878507e-14
+    8.8928864934219529e-14
 
     We now constrain the variables, in such a way that the previous solution
     becomes infeasible. Specifically, we require that ``x[1] >= 1.5``, and
@@ -634,7 +636,7 @@ def least_squares(
     >>> res_3 = least_squares(fun_broyden, x0_broyden,
     ...                       jac_sparsity=sparsity_broyden(n))
     >>> res_3.cost
-    4.5687161966109073e-23
+    4.5687069299604613e-23
     >>> res_3.optimality
     1.1650454296851518e-11
 
@@ -709,6 +711,30 @@ def least_squares(
     >>> plt.ylabel("y")
     >>> plt.legend()
     >>> plt.show()
+
+    In the next example, we show how complex-valued residual functions of
+    complex variables can be optimized with ``least_squares()``. Consider the
+    following function:
+
+    >>> def f(z):
+    ...     return z - (0.5 + 0.5j)
+
+    We wrap it into a function of real variables that returns real residuals
+    by simply handling the real and imaginary parts as independent variables:
+
+    >>> def f_wrap(x):
+    ...     fx = f(x[0] + 1j*x[1])
+    ...     return np.array([fx.real, fx.imag])
+
+    Thus, instead of the original m-dimensional complex function of n complex
+    variables we optimize a 2m-dimensional real function of 2n real variables:
+
+    >>> from scipy.optimize import least_squares
+    >>> res_wrapped = least_squares(f_wrap, (0.1, 0.1), bounds=([0, 0], [1, 1]))
+    >>> z = res_wrapped.x[0] + res_wrapped.x[1]*1j
+    >>> z
+    (0.49999999999925893+0.49999999999925893j)
+
     """
     if method not in ['trf', 'dogbox', 'lm']:
         raise ValueError("`method` must be 'trf', 'dogbox' or 'lm'.")
@@ -736,6 +762,9 @@ def least_squares(
     if max_nfev is not None and max_nfev <= 0:
         raise ValueError("`max_nfev` must be None or positive integer.")
 
+    if np.iscomplexobj(x0):
+        raise ValueError("`x0` must be real.")
+
     x0 = np.atleast_1d(x0).astype(float)
 
     if x0.ndim > 1:
@@ -750,7 +779,7 @@ def least_squares(
         raise ValueError("Inconsistent shapes between bounds and `x0`.")
 
     if np.any(lb >= ub):
-        raise ValueError("Each lower bound mush be strictly less than each "
+        raise ValueError("Each lower bound must be strictly less than each "
                          "upper bound.")
 
     if not in_bounds(x0, lb, ub):
@@ -893,7 +922,7 @@ def least_squares(
 
     if verbose >= 1:
         print(result.message)
-        print("Function evaluations: {0}, initial cost: {1:.4e}, final cost "
+        print("Function evaluations {0}, initial cost {1:.4e}, final cost "
               "{2:.4e}, first-order optimality {3:.2e}."
               .format(result.nfev, initial_cost, result.cost,
                       result.optimality))
